@@ -24,4 +24,7 @@
 }
 
 # will store information about formats that we can read
-.fileformats <- new.env()
\ No newline at end of file
+.fileformats <- new.env()
+
+# Will store stack of plotted rgl objects, ready for popping
+.plotted3d <- new.env()
