@@ -17,7 +17,20 @@ test_that("we can read im3d files",{
   expect_is(d0<-read.im3d(amfile, ReadData=FALSE), 'im3d')
   expect_equivalent(dim(d0), c(154L, 154L, 87L))
   
+  amfilenoam=tempfile()
+  file.copy(normalizePath(amfile),amfilenoam)
+  on.exit(unlink(amfilenoam))
+  expect_equal(d,read.im3d(amfilenoam))
+  
   expect_error(read.im3d("testdata/nrrd/LHMask.rhubarb"))
+  
+  v3drawfile1ch='testdata/v3draw/L1DS1_crop_straight_crop_ch1.v3draw'
+  v3drawfile2ch='testdata/v3draw/L1DS1_crop_straight_crop.v3draw'
+  v3drawfile2chslice='testdata/v3draw/L1DS1_crop_straight_crop_slice.v3draw'
+  
+  expect_error(read.im3d(v3drawfile2ch))
+  expect_equal(x<-read.im3d(v3drawfile2ch, chan=1), y<-read.im3d(v3drawfile1ch))
+  expect_equal(x[,,1], read.im3d(v3drawfile2chslice)[,,1])
 })
 
 test_that("round trip test for im3d is successful",{
@@ -86,11 +99,12 @@ test_that("dim, voxdims and boundingbox work",{
   
   expect_equal(voxdims(d), c(1.4, 1.4, 1.4))
   
-  bb_base=structure(c(0, 68.6, 0, 68.6, 0, 68.6), .Dim = 2:3)
+  bb_base=structure(c(0, 68.6, 0, 68.6, 0, 68.6), .Dim = 2:3, class='boundingbox')
   expect_equal(boundingbox(d), bb_base)
   expect_equal(boundingbox.character("testdata/nrrd/LHMask.nrrd"), bb_base)
   
-  expect_null(boundingbox(im3d(dims=c(2,3,4))))
+  bbdf=as.data.frame(unclass(bb_base))
+  expect_equal(boundingbox(bbdf),bb_base)
   
   expect_is(am<-read.im3d("testdata/amira/VerySmallLabelField.am", 
                           SimplifyAttributes=TRUE), 'im3d')
@@ -98,7 +112,8 @@ test_that("dim, voxdims and boundingbox work",{
   expect_equal(voxdims(am),c(0.5,0.5,2))
   # somewhat oddly, Amira decides that if dim=1 for any axis, the bounding
   # box will not be 0 or infinite, but the size that would be expected for dim=2
-  expect_equal(boundingbox(am),structure(c(0, 0.5, 0, 0.5, 0, 2), .Dim = 2:3))
+  expect_equal(boundingbox(am),structure(c(0, 0.5, 0, 0.5, 0, 2), .Dim = 2:3,
+                                         class='boundingbox'))
   
   expect_is(nrrd<-read.im3d("testdata/amira/VerySmallLabelField.nrrd",
                             SimplifyAttributes=TRUE), 'im3d')
@@ -114,6 +129,10 @@ test_that("dim, voxdims and boundingbox work",{
     "testdata/amira/VerySmallLabelField.am", SimplifyAttributes=TRUE)))
   # ... and again
   expect_equal(nrrdraw, amraw)
+  
+  kcs20bb=structure(c(284.594, 404.6951, 24.1869, 122.9557, 21.4379, 102.8015
+  ), .Dim = 2:3, class = "boundingbox")
+  expect_equal(boundingbox(kcs20), kcs20bb, tol=1e-4)
 })
 
 context("im3d flip, slice and projection")
